#define Program start
globalvar canvas, brush_radius, brush_radius_min, brush_radius_max, camera_x, camera_y, camera_oldx, camera_oldy, camera_zoom;
globalvar begin_scroll,scrolling,scroll_startx,scroll_starty,scroll_endx,scroll_endy;
globalvar brush_speed, brush_speed_min, brush_speed_max, brush_draw_counter;
globalvar droplet_radius, droplet_radius_min, droplet_radius_max;
globalvar mouse_handler;
globalvar color_left, color_right;
globalvar autosave_filename, folder_name, autosave_interval, save_filename;
globalvar image;

// Configuration
folder_name = "Spray Paint config"
autosave_filename = "Spray_Paint_autosave"
autosave_interval = 20 // Seconds
save_filename = ""

mouse_handler = 0 // Tracks which object is receiving mouse input
begin_scroll = false
scrolling = false

scroll_startx = 0
scroll_starty = 0
scroll_endx = 0
scroll_endy = 0

width=room_width
height=room_height-121 // Leave some space for the top UI. You'll see this number all throughout the program. I don't know why I didn't make it a global.
canvas = surface_create(width,height)

// Fill the canvas with white
canvas_background_color=c_white
surface_set_target(canvas)
draw_rectangle_color(0,0,surface_get_width(canvas),surface_get_height(canvas),canvas_background_color,canvas_background_color,canvas_background_color,canvas_background_color,false)
surface_reset_target()

// Init colors
color_left = c_purple
color_right = c_white

brush_radius = 100
brush_radius_min = 0
brush_radius_max = sqrt(power(width,2)+power(height,2))*0.6 // just large enough to cover the image

brush_speed = 3 // In seconds to fill the circle. It doesn't always fill this fast, because randomness
brush_speed_min=0.01
brush_speed_max=10
brush_draw_counter = 0 // Used when drawing at super slow speeds, like sub 1 droplet per frame. This keeps track how many frames to wait before drawing again

droplet_radius = 0.5
droplet_radius_min = 0.5
droplet_radius_max = 20

camera_x = 0
camera_y = -121
camera_zoom = 1

draw_set_circle_precision(64) // nice, crispy circles

// Set up the autosave folder
if not directory_exists(program_directory+"\"+folder_name) {
    directory_create(program_directory+"\"+folder_name)
}

// Dialog box configuration
message_background(dialog_background)
message_button(dialog_button)
message_text_font("Comic Sans MS",12,c_black,0)
message_button_font("Comic Sans MS",12,c_black,0)
message_input_font("Comic Sans MS",12,c_black,0)
message_input_color(c_silver)





#define Brush
var nx,ny,bx,by,r,theta,width,height,mx,my

if keyboard_check(219) { // The "[" key
    if (brush_radius>brush_radius_min) brush_radius-=1
}
if keyboard_check(221) { // The "]" key
    if (brush_radius<brush_radius_max) brush_radius+=1
}

// Release the mouse if it's in the UI area
if mouse_y<121 and mouse_handler==canvas {
    mouse_handler=-1
}
if mouse_y>121 && mouse_handler == -1 {
    mouse_handler = canvas
}
// Drawing time!

if (mouse_check_button(mb_left) || mouse_check_button(mb_right)) && !scrolling && mouse_handler==canvas { // aaaaaaaaa 121
    // draw a random pixel within the brush radius from the mouse
    // Rework this block
    if brush_speed>5 { // In seconds
        if brush_draw_counter==-1 {
            brush_draw_counter=floor(brush_speed)
        } else {
            brush_draw_counter -= 1
        }
        if brush_draw_counter == 0 {
            iterations = 10
        } else {
            iterations = 0
        }
    } else {
        brush_draw_counter=-1
        iterations = floor(pi*power(brush_radius,2)/60/brush_speed)
    }

    nx=mouse_x/camera_zoom+camera_x/camera_zoom // Convert from screen camera coordinates to canvas coordinates. I don't really know how this works, i just sort of figured it out xD
    ny=mouse_y/camera_zoom+camera_y/camera_zoom
    for (i=0;i<iterations;i+=1) {
        // This segment by aioobe on stackoverflow.com/a/50746409
        r = brush_radius * sqrt(random(1))
        theta = random(1) * 2 * pi
        
        bx = nx + r * cos(theta)
        by = ny + r * sin(theta)
        // End segment
        
        if mouse_check_button(mb_left) && !mouse_check_button(mb_right) {
            draw_circle_color(bx,by,droplet_radius,color_left,color_left,false)
        }
        if !mouse_check_button(mb_left) && mouse_check_button(mb_right) {
            draw_circle_color(bx,by,droplet_radius,color_right,color_right,false)
        }
        if mouse_check_button(mb_left) && mouse_check_button(mb_right) {
            if random(1)>0.5 {
                draw_circle_color(bx,by,droplet_radius,color_left,color_left,false)
            } else {
                draw_circle_color(bx,by,droplet_radius,color_right,color_right,false)
            }
        }
        //draw_circle_color(nx,ny,3,c_white,c_white,false)
    }
}

#define Main
var zoom_in, zoom_out;
zoom_in = false
zoom_out = false

if keyboard_check(vk_control) and keyboard_check_pressed(ord('S')) {
    // Save the image
    script_execute(save_canvas)
    //surface_save(canvas,"Testing 1 2 3.png")
    //show_message("Saved! :D")
}
if (keyboard_check_pressed(187)) or mouse_wheel_up() { // Keycode 187 is the =/+ key
   zoom_in = true
}
if (keyboard_check_pressed(189)) or mouse_wheel_down() { // Keycode 189 is the -/_ key
   zoom_out = true
}

// This script also handles panning and zooming the canvas

if zoom_in {
    // Get the mouse coordinate and convert it to canvas coordinates
    // Then it takes the difference from the current zoom level and the new zoom level and adjusts the camera offset so that the mouse stays in the same canvas spot
    mx=mouse_x/camera_zoom+camera_x/camera_zoom
    my=mouse_y/camera_zoom+camera_y/camera_zoom

    camera_x-=(mx*camera_zoom-(mx*(camera_zoom+0.1)))
    camera_y-=(my*camera_zoom-(my*(camera_zoom+0.1)))
    camera_zoom+=0.1
}
if zoom_out {
    // Same thing here
    mx=mouse_x/camera_zoom+camera_x/camera_zoom
    my=mouse_y/camera_zoom+camera_y/camera_zoom

    camera_x-=(mx*camera_zoom-(mx*(camera_zoom-0.1)))
    camera_y-=(my*camera_zoom-(my*(camera_zoom-0.1)))
    camera_zoom-=0.1
}
if camera_zoom<0 or camera_zoom>6 {
    script_execute(center_canvas)
}

if (keyboard_check(vk_space) && mouse_check_button(mb_left)) || mouse_check_button(mb_middle) {
    scrolling = true
    if begin_scroll==false {
        scroll_startx = mouse_x
        scroll_starty = mouse_y
        begin_scroll=true
    }
    scroll_endx=mouse_x
    scroll_endy=mouse_y
    camera_x=camera_oldx+scroll_startx-scroll_endx
    camera_y=camera_oldy+scroll_starty-scroll_endy
} else {
    camera_oldx = camera_x
    camera_oldy = camera_y
    begin_scroll=false
    scrolling=false
}

#define Rainbow_rect
// This script is unused. It was going to make the color wheel area of the color picker, but I didn't use it because it's super slow.

globalvar rainbow;

hue=0
rainbow=surface_create(255,255)
surface_set_target(rainbow)
for (cy=0;cy<10;cy+=1){
    hue=0
    for (cx=0;cx<255;cx+=1) {
        draw_point_color(make_color_hsv(hue,255,255),cx,cy)
        hue+=1
    }
}
surface_reset_target()

#define autosave
// Do the autosave, but only if the user isn't drawing, to avoid lag.
if not mouse_check_button(mb_left) and not mouse_check_button(mb_right) {
    surface_save(canvas,program_directory+"\"+folder_name+"\"+autosave_filename+".png")
    //show_message(program_directory)
}

#define load_image
if surface_exists(canvas){
    surface_free(canvas)
}
if file_exists(argument0) {
    image=background_add(argument0,false,false)
    canvas=surface_create(background_get_width(image),background_get_height(image))
    surface_set_target(canvas)
    draw_background(image,0,0)
    script_execute(center_canvas)
}





#define save_canvas
new_filename="-1"
should_overwrite=false
counter = 0
valid_save_name=false

while save_filename=="" and counter<2 {
    new_filename=get_save_filename("PNG|*.png","My epic drawing")
    if new_filename=="" {
        counter=2
    } else {
        //new_filename=get_string("Save path + name (only .png supported). (like 'C:\Users\[you]\Documents\my epic drawing.png')","C:\")
        // ^ Lol, before I found the file selection dialog
        if file_exists(new_filename) {
            if show_question("This file exists. Overwrite?") == true {
                valid_save_name=true
                save_filename = new_filename + ".png"
            }
        } else {
            save_filename = new_filename + ".png"
            valid_save_name=true
        }
    }
    counter+=1
}
if valid_save_name {
    surface_save(canvas,save_filename)
    if not file_exists(save_filename) {
        show_message("Error saving.")
        return 1
    }
}
return 0

#define new_canvas
if show_question("This will delete your current drawing. Continue?") == true {
    width=get_integer("Width of the new drawing:",room_width)
    height=get_integer("Height of the new drawing",room_height)
    //surface_free(canvas)
    canvas=surface_create(width,height)
    brush_radius_max = sqrt(power(width,2)+power(height,2))*0.6 // Just large enough to cover the image
    camera_y=-121
    camera_x=0
    camera_zoom=1
    // Fill the canvas with white
    canvas_background_color=c_white
    surface_set_target(canvas)
    draw_rectangle_color(0,0,surface_get_width(canvas),surface_get_height(canvas),canvas_background_color,canvas_background_color,canvas_background_color,canvas_background_color,false)
    surface_reset_target()
    script_execute(center_canvas)
}

#define user_load_image
if show_message_ext("This will erase your current drawing. Continue?","No","Maybe","Yes")==3 {
    path = get_open_filename("All files|*;|PNG|*.png","")
    
    if file_exists(path) {
        surface_free(canvas)
        image=background_add(path,false,false)
        canvas=surface_create(background_get_width(image),background_get_height(image))
        surface_set_target(canvas)
        draw_background(image,0,0)
        script_execute(center_canvas)
    }
}


#define center_canvas
// Centers the canvas in the viewport
camera_x = surface_get_width(canvas)/2 - room_width/2
camera_y = (surface_get_height(canvas)/2 - room_height/2)-(121/2) // aaaaaaa
camera_zoom = 1

